@@ -5,31 +5,37 @@
 void Process::deactivate(bool restart) { sim->unschedule(this); }
 
 void Generator::run() {
-  // get the delay for the next arrival
-  double delay = Rcpp::as<double>(dist());
-  if (delay < 0) return;
+  // get the delay for the next (n) arrival(s)
+  Rcpp::NumericVector delays = dist();
+  int n = delays.size();
+  double delay = 0;
   
-  // format the name and create the next arrival
-  char numstr[21];
-  sprintf(numstr, "%d", count);
-  Arrival* arrival = new Arrival(sim, name + numstr, is_monitored(), first_activity, this);
-  
-  if (sim->verbose) Rcpp::Rcout << 
-    FMT_0 << sim->now() << " |" << FMT_11 << "generator: " << FMT_12 << name << "|" << 
-    FMT_21 << "new: " << FMT_22 << (name + numstr) << "| " << (sim->now() + delay) << std::endl;
-  
-  // schedule the arrival and the generator itself
-  sim->schedule(delay, arrival, count);
+  for(int i = 0; i < n; ++i) {
+    if (delays[i] < 0) return;
+    delay += delays[i];
+    
+    // format the name and create the next arrival
+    char numstr[21];
+    sprintf(numstr, "%d", count);
+    Arrival* arrival = new Arrival(sim, name + numstr, is_monitored(), first_activity, this);
+    
+    if (sim->verbose) Rcpp::Rcout << 
+      FMT(10, right) << sim->now() << " |" << FMT(12, right) << "generator: " << FMT(15, left) << name << "|" << 
+      FMT(12, right) << "new: " << FMT(15, left) << (name + numstr) << "| " << (sim->now() + delay) << std::endl;
+    
+    // schedule the arrival
+    sim->schedule(delay, arrival, count);
+    count++;
+  }
+  // schedule the generator
   sim->schedule(delay, this, PRIORITY_GENERATOR);
-  
-  count++;
 }
 
 void Manager::run() {
   if (!sim->now() && duration[index]) goto finish;
   if (sim->verbose) Rcpp::Rcout <<
-    FMT_0 << sim->now() << " |" << FMT_11 << "manager: " << FMT_12 << name << "|" << 
-    FMT_21 << "parameter: " << FMT_22 << param << "| " << value[index] << std::endl;
+    FMT(10, right) << sim->now() << " |" << FMT(12, right) << "manager: " << FMT(15, left) << name << "|" << 
+    FMT(12, right) << "parameter: " << FMT(15, left) << param << "| " << value[index] << std::endl;
   
   set(value[index]);
   index++;
@@ -44,14 +50,28 @@ end:
   return;
 }
 
+void DelayedTask::run() {
+  if (sim->verbose) Rcpp::Rcout <<
+    FMT(10, right) << sim->now() << " |" << FMT(12, right) << "task: " << FMT(15, left) << name << "|" << 
+    FMT(12, right) << " " << FMT(15, left) << " " << "| " << std::endl;
+  
+  task();
+  delete this;
+}
+
 void Arrival::run() {
   double delay;
   
   if (!activity) goto finish;
   if (lifetime.start < 0) lifetime.start = sim->now();
-  if (sim->verbose) Rcpp::Rcout <<
-    FMT_0 << sim->now() << " |" << FMT_11 << "arrival: " << FMT_12 << name << "|" << 
-    FMT_21 << "activity: " << FMT_22 << activity->name << "| " << activity->resource << std::endl;
+  
+  if (sim->verbose) {
+    Rcpp::Rcout <<
+      FMT(10, right) << sim->now() << " |" << 
+      FMT(12, right) << "arrival: " << FMT(15, left) << name << "|" << 
+      FMT(12, right) << "activity: " << FMT(15, left) << activity->name << "| ";
+    activity->print(0, true);
+  }
   
   delay = activity->run(this);
   if (delay == REJECTED) goto end;
