@@ -12,7 +12,7 @@ test_that("resources are seized/released as expected", {
     timeout(1) %>%
     seize("dummy", 1)
   
-  env <- simmer() %>%
+  env <- simmer(verbose=TRUE) %>%
     add_resource("dummy", 3, 0) %>%
     add_generator("arrival", t0, at(0))
   
@@ -20,12 +20,17 @@ test_that("resources are seized/released as expected", {
   expect_equal(env %>% get_server_count("dummy"), 1)
   env %>% onestep() %>% onestep()
   expect_equal(env %>% get_server_count("dummy"), 3)
-  env %>% onestep() %>% onestep()
+  env %>% onestep() %>% onestep() %>% onestep()
   expect_equal(env %>% get_server_count("dummy"), 2)
   env %>% onestep() %>% onestep()
   expect_equal(env %>% get_server_count("dummy"), 0)
 })
 
+test_that("preemptible < priority shows a warning", {
+  expect_warning(create_trajectory() %>%
+    seize("dummy", 1, priority=3, preemptible=1))
+})
+
 test_that("incorrect types fail", {
   expect_error(create_trajectory() %>% seize(0, 0))
   expect_error(create_trajectory() %>% release(0, 0))
