@@ -2,8 +2,8 @@ context("set attributes")
 
 test_that("only valid types can be passed to functions", {
   expect_error(create_trajectory()%>%set_attribute("test", "string_value"))
-  expect_error(not(create_trajectory()%>%set_attribute("test", 3)))
-  expect_error(not(create_trajectory()%>%set_attribute("test", function() 3)))
+  expect_error(create_trajectory()%>%set_attribute("test", 3), NA)
+  expect_error(create_trajectory()%>%set_attribute("test", function() 3), NA)
   
 })
 
@@ -24,10 +24,10 @@ test_that("an attribute is correctly set and returned to a function that needs i
 
 test_that("attributes can be correctly retrieved using get_mon_attributes()", {
   t0 <- create_trajectory() %>%
-    set_attribute("test", 123)
+    set_attribute("test", function() 123)
   
   env<-
-    simmer() %>%
+    simmer(verbose=TRUE) %>%
     add_generator("entity", t0, at(0), mon=2) %>%
     run()
   
